from typing import List, Set

from fastapi import HTTPException
from starlette import status

from ucsschool.apis.utils import get_logger
from ucsschool.kelvin.client import InvalidRequest, KelvinObject, KelvinResource


async def search_kelvin_obj(resource: KelvinResource, **search_args) -> KelvinObject:
    """
    Searches for an object via the Kelvin API and returns it.

    :param resource: The KelvinResource to use for searching for the object
    :param search_args: search arguments passed to the resources search method
    :return: The found KelvinObject
    :raises HTTPException: If the request failed, if no object or if more than one object was found
    """
    logger = get_logger()
    try:
        kelvin_objects = [kelvin_object async for kelvin_object in resource.search(**search_args)]
    except InvalidRequest as exc:
        raise HTTPException(status_code=exc.status, detail=exc.reason) from exc
    if len(kelvin_objects) == 0:
        logger.debug(
            f"{resource.Meta.kelvin_object.__name__} not found with search params: {search_args!r}."
        )
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Object not found.",
        )
    elif len(kelvin_objects) > 1:
        logger.error(
            f"Multiple {resource.Meta.kelvin_object.__name__} objects found with search params: "
            f"{search_args!r} "
        )
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail="More than one object of this resource was found.",
        )
    return kelvin_objects[0]


def remove_prefix(value: str, prefix: str) -> str:
    """
    Removes the given prefix from the given string.

    >>> remove_prefix("prefix-some_string", "prefix-")
    'some_string'

    :param value: The string to remove the prefix from
    :param prefix: The prefix to remove from the string
    :return: The string without the prefix
    :raises ValueError: If the given string does not start with the given prefix
    """
    if not value.startswith(prefix):
        raise ValueError(f"The value {value} must begin with the prefix: {prefix}")
    return value.removeprefix(prefix)


def get_roles_for_school(roles: List[str], school: str) -> Set[str]:
    """
    Takes a list of ucsschool_roles and returns a list of user roles for the given school.

    >>> get_roles_for_school(["teacher:school:School1", "student:school:School2"], "School1")
    {'teacher'}

    :param roles: The list of ucsschool_roles to filter
    :param school: The school to filter the roles for
    :return: The list of user roles for the given school
    :raises ValueError: If any of the role strings is malformed
    """
    filtered_roles = set()
    for role in roles:
        role_parts = role.split(":")
        if len(role_parts) != 3 or not all(role_parts):
            raise ValueError(f"The role {role} is malformed!")
        if (
            role_parts[1] == "school"
            and role_parts[2] == school
            and role_parts[0] in ("student", "teacher", "staff")
        ):
            filtered_roles.add(role_parts[0])
    return filtered_roles


def get_source_uid_for_authority(school_authority: str) -> str:
    return school_authority
